# general rule for shifted vs truncated
shifted
# general rule tailcorrections
no
# number of defined interactions
3
# type interaction
CH4_sp3      LENNARD_JONES     158.5     3.72
Si           LENNARD_JONES     0         1.0
O            LENNARD_JONES     83.43     3.2
# general mixing rule for Lennard-Jones
Lorentz-Berthelot
