# critical constants: Temperature [T], Pressure [Pa], and Acentric factor [-]
190.564
4599200.0
0.01142
# Number Of Atoms
1
# Number Of Groups
1
# Alkane-group
flexible
# number of atoms
1
# atomic positions
0 CH4_sp3
# Chiral centers Bond  BondDipoles Bend  UrayBradley InvBend  Torsion Imp. Torsion Bond/Bond Bond/Bend Bend/Bend Bond/Torsion Bend/Torsion IntraVDW Intra ch-ch Intra ch-bd Intra bd-bd
               0    0            0    0            0       0        0            0         0         0         0            0            0        0           0           0           0
# Number of config moves
0
